#include "stdafx.h"
#include "resource.h"
#include <stdio.h>
#include <mmsystem.h>
#include <time.h>
#include <windows.h>
#include <tchar.h>  // For TCHAR definitions

// for sound
#pragma comment(lib, "winmm.lib")

#define MAX_LOADSTRING 100
#define TIMER_ID 1
#define BATTERY_UPDATE_TIMER_ID 2
#define TIMER_INTERVAL 1000 // 1 second
#define BATTERY_UPDATE_INTERVAL 5000 // 5 seconds

#define WINDOW_WIDTH 300
#define WINDOW_HEIGHT 140

//timer length when batt status is 1%
#define BATTERY_CRITICAL_TIME 3600 // 1 hour in seconds

//stats at time of creating this (7/25/24)
//batt % reaches 1 at 1:20:00
//batt runs an additonal 1:20:00
//total runtime; 2:40:00

//minimum allowed starting charge %
#define BATTERY_MIN_PERCENT 95 

// Global Variables:
HINSTANCE hInst; // current instance
TCHAR szTitle[MAX_LOADSTRING]; // app title bar text
TCHAR szWindowClass[MAX_LOADSTRING]; // window title bar text

bool timerStarted = false;
bool countdownStarted = false;
time_t timerEndTime;
bool alertShown = false;
bool acStatusPrevious = true; // Track previous AC line status (initialized to true assuming it starts connected)

// Forward declarations of functions included in this code module:
ATOM MyRegisterClass(HINSTANCE hInstance);
BOOL InitInstance(HINSTANCE, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK About(HWND, UINT, WPARAM, LPARAM);

void StartTimer();
void ResetTimer();
void GetBatteryStatus(TCHAR * szStatus, int bufferSize);

int APIENTRY WinMain(HINSTANCE hInstance,
  HINSTANCE hPrevInstance,
  LPSTR lpCmdLine,
  int nCmdShow) {
  MSG msg;
  HACCEL hAccelTable;

  // Initialize global strings
  LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
  LoadString(hInstance, IDC_BATTMON, szWindowClass, MAX_LOADSTRING);
  MyRegisterClass(hInstance);

  // Perform application initialization:
  if (!InitInstance(hInstance, nCmdShow)) {
	return FALSE;
  }

  hAccelTable = LoadAccelerators(hInstance, (LPCTSTR) IDC_BATTMON);

  // Main message loop:
  while (GetMessage( & msg, NULL, 0, 0)) {
    if (!TranslateAccelerator(msg.hwnd, hAccelTable, & msg)) {
      TranslateMessage( & msg);
      DispatchMessage( & msg);
    }
  }

  return (int) msg.wParam;
}

ATOM MyRegisterClass(HINSTANCE hInstance) {
  WNDCLASSEX wcex;

  wcex.cbSize = sizeof(WNDCLASSEX);

  wcex.style = CS_HREDRAW | CS_VREDRAW;
  wcex.lpfnWndProc = (WNDPROC) WndProc;
  wcex.cbClsExtra = 0;
  wcex.cbWndExtra = 0;
  wcex.hInstance = hInstance;
  wcex.hIcon = LoadIcon(hInstance, (LPCTSTR) IDI_BATTMON);
  wcex.hCursor = LoadCursor(NULL, IDC_ARROW);
  wcex.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
  wcex.lpszMenuName = (LPCSTR) IDC_BATTMON;
  wcex.lpszClassName = szWindowClass;
  wcex.hIconSm = LoadIcon(wcex.hInstance, (LPCTSTR) IDI_SMALL);

  return RegisterClassEx( & wcex);
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow) {
  HWND hWnd;

  hInst = hInstance; // Store instance handle in our global variable

  // Calculate the window size to account for borders and title bar
  RECT rect = {
    0,
    0,
    WINDOW_WIDTH,
    WINDOW_HEIGHT
  };
  AdjustWindowRect( & rect, WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX, FALSE);

  hWnd = CreateWindow(
    szWindowClass,
    szTitle,
    WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX,
    CW_USEDEFAULT, CW_USEDEFAULT,
    rect.right - rect.left, rect.bottom - rect.top,
    NULL,
    NULL,
    hInstance,
    NULL
  );

  if (!hWnd) {
    return FALSE;
  }

  ShowWindow(hWnd, nCmdShow);
  UpdateWindow(hWnd);

  // Start the battery update timer
  SetTimer(hWnd, BATTERY_UPDATE_TIMER_ID, BATTERY_UPDATE_INTERVAL, NULL);

  return TRUE;
}

void StartTimer() {
  timerStarted = true;
  timerEndTime = time(NULL) + BATTERY_CRITICAL_TIME;
}

void ResetTimer() {
  timerStarted = false;
  countdownStarted = false;
  alertShown = false;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
  static TCHAR szBatteryStatus[256];
  int wmId, wmEvent;
  PAINTSTRUCT ps;
  HDC hdc;

  switch (message) {
  case WM_COMMAND:
    wmId = LOWORD(wParam);
    wmEvent = HIWORD(wParam);
    // Parse the menu selections:
    switch (wmId) {
    case IDM_ABOUT:
      DialogBox(hInst, (LPCTSTR) IDD_ABOUTBOX, hWnd, (DLGPROC) About);
      break;
    case IDM_EXIT:
      DestroyWindow(hWnd);
      break;
    default:
      return DefWindowProc(hWnd, message, wParam, lParam);
    }
    break;
  case WM_PAINT:
    hdc = BeginPaint(hWnd, & ps);
    RECT rt;
    GetClientRect(hWnd, & rt);
    DrawText(hdc, szBatteryStatus, -1, & rt, DT_CENTER | DT_VCENTER | DT_WORDBREAK);
    EndPaint(hWnd, & ps);
    break;
  case WM_TIMER:
    if (wParam == TIMER_ID) {
      // Handle the existing timer (if needed)
    } else if (wParam == BATTERY_UPDATE_TIMER_ID) {
      GetBatteryStatus(szBatteryStatus, sizeof(szBatteryStatus) / sizeof(TCHAR));
      InvalidateRect(hWnd, NULL, TRUE);
    }
    break;
  case WM_DESTROY:
    KillTimer(hWnd, TIMER_ID);
    KillTimer(hWnd, BATTERY_UPDATE_TIMER_ID);
    PostQuitMessage(0);
    break;
  default:
    return DefWindowProc(hWnd, message, wParam, lParam);
  }
  return 0;
}

// Message handler for about box.
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
  switch (message) {
  case WM_INITDIALOG:
    return TRUE;

  case WM_COMMAND:
    if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
      EndDialog(hDlg, LOWORD(wParam));
      return TRUE;
    }
    break;
  }
  return FALSE;
}

void GetBatteryStatus(TCHAR * szStatus, int bufferSize) {
  SYSTEM_POWER_STATUS sps;

  if (GetSystemPowerStatus( & sps)) {
    static bool countdownAllowed = false; // Flag to determine if countdown is allowed
	static bool alerted = false;

    // Check if AC power was previously on and now it's off
    if (acStatusPrevious && sps.ACLineStatus == 0) {
      if (sps.BatteryLifePercent <= BATTERY_MIN_PERCENT) {
        // Battery level is 1% or less, show alert and do not start the timer
        if (!timerStarted && !alerted) {
          alerted = true;
          MessageBox(NULL, _T("Must begin with a fully charged battery. Once system status depletes from 100% to 1% the adjusted remaining time will engage."), _T("Battery Alert"), MB_OK | MB_ICONWARNING);
          ResetTimer(); // Ensure the timer is reset if we are not starting it

        }
      } else {
        // Battery level is above required%, allow countdown to start
        countdownAllowed = true;
      }
    } else if (!acStatusPrevious && sps.ACLineStatus == 1) {
      // AC power is restored, reset the countdown
      ResetTimer();
      alerted = false;
    }

    // Update the previous AC line status
    acStatusPrevious = (sps.ACLineStatus == 1);

    int remainingTime = 0;
    if (countdownAllowed && sps.ACLineStatus == 0) {
		if (sps.BatteryLifePercent <= 1) { //start timer when batt stats drops to 1%
        if (!timerStarted) {
          StartTimer();
        }

        remainingTime = (int) difftime(timerEndTime, time(NULL));
        if (remainingTime <= 0) {
          remainingTime = 0;
          timerStarted = false;
          countdownStarted = false;
        }
        if (remainingTime <= 600 && !alertShown){ // 10 minutes = 600 seconds
			alertShown = true;
			PlaySound(TEXT("SystemHand"), NULL, SND_ALIAS | SND_ASYNC);
			MessageBox(NULL, _T("Battery is getting low. Plug in soon."), _T("Battery Alert"), MB_OK | MB_ICONWARNING);
        }
      } else {
        remainingTime = 0; // If battery is above 1%, no countdown should be active
      }
    } else {
      remainingTime = 0; // If countdown is not allowed or AC power is on
    }

    // Format the status string
    if (sps.ACLineStatus == 1) {
      // If plugged into AC, do not show time remaining
      _stprintf(szStatus, _T("\nPower Source: %s\n\nSystem Battery Status: %d%%"),
        (sps.ACLineStatus == 1) ? _T("A/C") : _T("Battery"),
        sps.BatteryLifePercent);
    } else {
      // If running on battery, show time remaining only if countdown is allowed
      int hours = remainingTime / 3600;
      int minutes = (remainingTime % 3600) / 60;

      if (countdownAllowed) {
        if (hours == 0 && minutes == 0) {
          _stprintf(szStatus, _T("\nPower Source: %s\n\nSystem Battery Status: %d%%\n\nAdjusted Time Remaining: --:--"),
            (sps.ACLineStatus == 1) ? _T("A/C") : _T("Battery"),
            sps.BatteryLifePercent);
        } else {
          _stprintf(szStatus, _T("\nPower Source: %s\n\nSystem Battery Status: %d%%\n\nAdjusted Time Remaining: %02d:%02d"),
            (sps.ACLineStatus == 1) ? _T("A/C") : _T("Battery"),
            sps.BatteryLifePercent,
            hours,
            minutes);
        }
      } else {
        _stprintf(szStatus, _T("\nPower Source: %s\n\nSystem Battery Status: %d%%\n\nTime Remaining: Disabled"),
          (sps.ACLineStatus == 1) ? _T("A/C") : _T("Battery"),
          sps.BatteryLifePercent);
      }
    }
  } else {
    _tcscpy(szStatus, _T("Unable to retrieve System Battery Status."));
  }
}