// Catch1K.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "resource.h"
#include <stdio.h>
#include <mmsystem.h>
#include <time.h>
#include <windows.h>

LARGE_INTEGER frequency;
double frameInterval = 0;  // Default to 0

const LARGE_INTEGER FAST_MACHINE_THRESHOLD = { 10000000 }; // Adjust this threshold based on your requirements


#pragma comment(lib, "winmm.lib")

//random gameplay sounds
const char* soundFiles[] = {
    "wav/s01.wav",
    "wav/s02.wav",
};

const int numSounds = sizeof(soundFiles) / sizeof(soundFiles[0]);

#define MAXLIVES 3

#define WINDOW_WIDTH 420
#define WINDOW_HEIGHT 300

#define MAX_LOADSTRING 100
#define WM_START_DRAWING (WM_USER + 1)

HINSTANCE hInst;
TCHAR szTitle[MAX_LOADSTRING];
TCHAR szWindowClass[MAX_LOADSTRING];
BOOL bDraw = FALSE;
BOOL bClosing = FALSE;  // New flag to indicate window is closing
BOOL initGame = true;
bool userWin = false;
bool paused = false;
int tries = 0;  // Tries counter
int lives = MAXLIVES;  // Lives counter

ATOM MyRegisterClass(HINSTANCE hInstance);
BOOL InitInstance(HINSTANCE, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK About(HWND, UINT, WPARAM, LPARAM);

void resetGame() {
	initGame=true;
	userWin = false;
	paused = false;
	tries = 0;
	lives = MAXLIVES;
	bDraw = false;
}

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
    MSG msg;
    HACCEL hAccelTable;

    LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
    LoadString(hInstance, IDC_CATCH1K, szWindowClass, MAX_LOADSTRING);
    MyRegisterClass(hInstance);

    if (!InitInstance(hInstance, nCmdShow)) 
    {
        return FALSE;
    }

	if (QueryPerformanceFrequency(&frequency) == 0) {
		// Handle error
	}

	// Set frameInterval based on the performance
	if (frequency.QuadPart > FAST_MACHINE_THRESHOLD.QuadPart) {
		frameInterval = 1.09;  // Frame interval for faster machines
	} else {
		frameInterval = 0;  // No throttling for slower machines
	}

    hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_CATCH1K);

    srand((unsigned int)time(NULL));

    while (GetMessage(&msg, NULL, 0, 0)) 
    {
        if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }

    return msg.wParam;
}

ATOM MyRegisterClass(HINSTANCE hInstance)
{
    WNDCLASSEX wcex;

    wcex.cbSize = sizeof(WNDCLASSEX); 

    wcex.style          = CS_HREDRAW | CS_VREDRAW;
    wcex.lpfnWndProc    = (WNDPROC)WndProc;
    wcex.cbClsExtra     = 0;
    wcex.cbWndExtra     = 0;
    wcex.hInstance      = hInstance;
    wcex.hIcon          = LoadIcon(hInstance, (LPCTSTR)IDI_CATCH1K);
    wcex.hCursor        = LoadCursor(NULL, IDC_ARROW);
    wcex.hbrBackground  = (HBRUSH)GetStockObject(BLACK_BRUSH);  // Set background to black
    wcex.lpszMenuName   = (LPCSTR)IDC_CATCH1K;
    wcex.lpszClassName  = szWindowClass;
    wcex.hIconSm        = LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

    return RegisterClassEx(&wcex);
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
   HWND hWnd;

   hInst = hInstance;

    // Calculate the window size to account for borders and title bar
    RECT rect = {0, 0, WINDOW_WIDTH, WINDOW_HEIGHT};
    AdjustWindowRect(&rect, WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX, FALSE);

    hWnd = CreateWindow(
        szWindowClass,
		szTitle,
        WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX,
        CW_USEDEFAULT, CW_USEDEFAULT, 
        rect.right - rect.left, rect.bottom - rect.top,
        NULL,
        NULL,
        hInstance,
        NULL
    );

	//hWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);
	//hWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, 0, 420, 300, NULL, NULL, hInstance, NULL);

   if (!hWnd)
   {
      return FALSE;
   }

   ShowWindow(hWnd, nCmdShow);
   UpdateWindow(hWnd);

   return TRUE;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    int wmId, wmEvent;
    PAINTSTRUCT ps;
    HDC hdc;
    static int drawCount = 0;
    static int sndcnt = 0;
    MSG msg;

    switch (message) 
    {
        case WM_COMMAND:
            wmId    = LOWORD(wParam); 
            wmEvent = HIWORD(wParam); 
            switch (wmId)
            {
                case IDM_ABOUT:
					DialogBox(hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
					break;
                case IDM_EXIT:
					DestroyWindow(hWnd);
					break;
				case IDM_RESTART:
					resetGame();
					drawCount = 0;
					///RECT rt;
					//GetClientRect(hWnd, &rt);
					//HBRUSH hBrush = CreateSolidBrush(RGB(0, 0, 0));
					//FillRect(hdc, &Rt, hBrush);
					//DeleteObject(hBrush);


					//PostMessage(hWnd, WM_PAINT, 0, 0);
					InvalidateRect(hWnd, NULL, FALSE);
					break;
                default:
                   return DefWindowProc(hWnd, message, wParam, lParam);
            }
            break;

        case WM_PAINT:
            hdc = BeginPaint(hWnd, &ps);
            if (!bDraw && !paused) {

				paused = true;
                // Draw the current counter and tries count when not drawing
                TCHAR szCounter[MAX_LOADSTRING];
                TCHAR szTries[MAX_LOADSTRING];
                TCHAR szLives[MAX_LOADSTRING];
				
				int range = abs(drawCount - 1000);

                if (drawCount == 1000) { //nailed it
                    wsprintf(szCounter, TEXT(".:::: Got it! ::::."));
					PlaySound("wav/win.wav", NULL, SND_FILENAME | SND_ASYNC);
					userWin=true;
                } else if (range <= 20) { //close
					if(lives < MAXLIVES) { //give a life back
						lives++;
					}
                    wsprintf(szCounter, TEXT("!!!! Close !!!!"));
					PlaySound("wav/cheer.wav", NULL, SND_FILENAME | SND_ASYNC);
                } else { //missed
					if(!initGame) {
						lives--;
						if(lives) {
							wsprintf(szCounter, TEXT("-----> Ouch! <-----"));
							PlaySound("wav/fail.wav", NULL, SND_FILENAME | SND_ASYNC);
						} else {
							wsprintf(szCounter, TEXT("-----> You Lose! Play again? <-----"));
							PlaySound("wav/end.wav", NULL, SND_FILENAME | SND_ASYNC);
						}
					} else { //restart 
						wsprintf(szCounter, TEXT("Hit a key to start. Stop on 1,000."));
						PlaySound("wav/start.wav", NULL, SND_FILENAME | SND_ASYNC);
					}
					
				}
                
                wsprintf(szTries, TEXT("Tries: %d"), tries);
                wsprintf(szLives, TEXT("Lives: %d"), lives);
                
                RECT rt;
                GetClientRect(hWnd, &rt);
                
                // Create a rectangle for the text background
				RECT textRect = rt;
				textRect.top = (rt.bottom - rt.top) / 2 - 50;  // Adjust these values to position the rectangle
				textRect.bottom = textRect.top + 40;  // Adjust height as needed
				textRect.left = rt.left + 60;
				textRect.right = rt.right - 60;

				// Create and select the black brush and pen
				HBRUSH hBrush = CreateSolidBrush(RGB(0, 0, 0));
				HPEN hPen = CreatePen(PS_SOLID, 1, RGB(0, 0, 0));
				HGDIOBJ hOldBrush = SelectObject(hdc, hBrush);
				HGDIOBJ hOldPen = SelectObject(hdc, hPen);

				// Draw the rounded rectangle for the main text background
				int cornerRadius = 20;  // Adjust this value to change the roundness of the corners
				RoundRect(hdc, textRect.left, textRect.top, textRect.right, textRect.bottom, cornerRadius, cornerRadius);

				// Restore the original brush and pen
				SelectObject(hdc, hOldBrush);
				SelectObject(hdc, hOldPen);

				// Set the text mode to transparent and draw the main text
				SetBkMode(hdc, TRANSPARENT);
				SetTextColor(hdc, RGB(255, 255, 255));  // White text
				DrawText(hdc, szCounter, -1, &textRect, DT_CENTER | DT_VCENTER | DT_SINGLELINE);

				// Draw the tries count below the main text
				RECT triesRect = textRect;
				triesRect.top += 50;  // Adjust position as needed
				triesRect.bottom += 50;

				// Select the black brush and pen again for the tries rectangle
				hOldBrush = SelectObject(hdc, hBrush);
				hOldPen = SelectObject(hdc, hPen);

				// Draw the rounded rectangle for the tries background
				RoundRect(hdc, triesRect.left, triesRect.top, triesRect.right, triesRect.bottom, cornerRadius, cornerRadius);

				// Restore the original brush and pen
				SelectObject(hdc, hOldBrush);
				SelectObject(hdc, hOldPen);

				// Draw the text for tries
				DrawText(hdc, szTries, -1, &triesRect, DT_CENTER | DT_VCENTER | DT_SINGLELINE);

				// Draw the lives count below the tries text
				RECT livesRect = triesRect;
				livesRect.top += 25;  // Adjust position as needed
				livesRect.bottom += 35;

				// Select the black brush and pen again for the lives rectangle
				hOldBrush = SelectObject(hdc, hBrush);
				hOldPen = SelectObject(hdc, hPen);

				// Draw the rounded rectangle for the lives background
				RoundRect(hdc, livesRect.left, livesRect.top, livesRect.right, livesRect.bottom, cornerRadius, cornerRadius);

				// Restore the original brush and pen
				SelectObject(hdc, hOldBrush);
				SelectObject(hdc, hOldPen);

				// Draw the text for lives
				DrawText(hdc, szLives, -1, &livesRect, DT_CENTER | DT_VCENTER | DT_SINGLELINE);

				// Delete the brush and pen
				DeleteObject(hBrush);
				DeleteObject(hPen);

            }
            EndPaint(hWnd, &ps);
            break;

        case WM_KEYDOWN:
            if (!bClosing) {  // Only toggle if not closing
                bDraw = !bDraw;
                if (bDraw) {
					paused = false;
					initGame = false;
					if(userWin == true) {
						tries=0;
						userWin = false;
					}
					if(!lives) { //reset game on a lose
						tries=0;
						lives = MAXLIVES;
					}
                    tries++;  // Increment tries counter
                    // MessageBox(hWnd, "Starting drawing", "Debug", MB_OK);  // Debug message box
                    drawCount = 0;  // Reset counter when starting
                    InvalidateRect(hWnd, NULL, TRUE);  // Clear the screen when starting
                    PostMessage(hWnd, WM_START_DRAWING, 0, 0);
                } else {
                    if (drawCount == 1000) { //they won
                        tries = 0;  // Reset tries counter if player stopped on 1000
                    }
                    // MessageBox(hWnd, "Stopping drawing", "Debug", MB_OK);  // Debug message box
                    InvalidateRect(hWnd, NULL, FALSE);  // Redraw the window to show the counter and tries count
                }
            }
            break;

		case WM_START_DRAWING:
			if (bDraw && !bClosing) {
				hdc = GetDC(hWnd);
				RECT rt;
				GetClientRect(hWnd, &rt);
				int width = rt.right;
				int height = rt.bottom;

				TCHAR counterText[MAX_LOADSTRING];
				RECT trect = rt;
				trect.top = 30; // text rect
				trect.bottom = 64; // Increased bottom to accommodate larger text
				trect.left = (width / 2) - 100; // Adjusted for larger text
				trect.right = (width / 2) + 100; // Adjusted for larger text
				
				// Set the background color to black
				SetBkColor(hdc, RGB(0, 0, 0)); // Black
				// Set text color to white
				SetTextColor(hdc, RGB(255, 255, 255)); // White
				// Set the background mode to opaque to ensure full coverage
				SetBkMode(hdc, OPAQUE);

				// Clear the text background area with the background color
				// HBRUSH hBrush = CreateSolidBrush(RGB(0, 0, 0));
				// FillRect(hdc, &trect, hBrush);
				// DeleteObject(hBrush);

				// Create a larger font
				/*
				HFONT hFont = CreateFont(
					34,            // Font height
					0,             // Font width (0 means default)
					0,             // Angle of the font
					0,             // Base line orientation
					FW_NORMAL,     // Font weight
					FALSE,         // Italic
					FALSE,         // Underline
					FALSE,         // Strikeout
					ANSI_CHARSET,  // Character set
					OUT_DEFAULT_PRECIS, // Output precision
					CLIP_DEFAULT_PRECIS, // Clipping precision
					DEFAULT_QUALITY, // Font quality
					DEFAULT_PITCH | FF_SWISS, // Pitch and family
					TEXT("Arial") // Font name
				);
				HFONT hOldFont = (HFONT)SelectObject(hdc, hFont);
				*/				

				// Variables for timing
				LARGE_INTEGER start, end;
				double elapsedTime;

				// Draw a batch of lines
				for (int i = 0; i < 1100 && !bClosing && bDraw; i++) {
					if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
						if (msg.message == WM_KEYDOWN) {
							bDraw = FALSE;
							InvalidateRect(hWnd, NULL, FALSE);
							break;
						}
						TranslateMessage(&msg);
						DispatchMessage(&msg);
					}

					// Start timing
					QueryPerformanceCounter(&start);

					COLORREF color = RGB(rand() % 256, rand() % 256, rand() % 256);
					HPEN hpen = CreatePen(PS_SOLID, 2, color);
					HPEN holdpen = (HPEN)SelectObject(hdc, hpen);

					MoveToEx(hdc, rand() % width, rand() % height, NULL);
					LineTo(hdc, rand() % width, rand() % height);

					SelectObject(hdc, holdpen);
					DeleteObject(hpen);

					drawCount++;

					//draw the counter text
						
					// Clear the text background area with the background color
					//HBRUSH hBrush = CreateSolidBrush(RGB(0, 0, 0));
					//FillRect(hdc, &trect, hBrush);
					//DeleteObject(hBrush);
				
					wsprintf(counterText, TEXT("- %04d -"), drawCount);
					DrawText(hdc, counterText, -1, &trect, DT_CENTER | DT_VCENTER | DT_SINGLELINE);

					if (sndcnt == 20) {
						int soundIndex = rand() % numSounds;
						PlaySound(soundFiles[soundIndex], NULL, SND_FILENAME | SND_ASYNC);
						sndcnt = 0;
					}
					sndcnt++;

					// End timing and calculate elapsed time
					QueryPerformanceCounter(&end);
					elapsedTime = (double)(end.QuadPart - start.QuadPart) * 1000.0 / frequency.QuadPart;

					// Sleep for the remaining frame time, if any
					if (elapsedTime < frameInterval) {
						Sleep((DWORD)(frameInterval - elapsedTime));
					}
				}

				// Restore the old font and delete the new font
				//SelectObject(hdc, hOldFont);
				//DeleteObject(hFont);

				ReleaseDC(hWnd, hdc);

				// Continue drawing if not stopped and not closing
				if (bDraw && !bClosing) {
					drawCount = 0; // reset counter
					PostMessage(hWnd, WM_START_DRAWING, 0, 0);
				}
			}
			break;

        case WM_CLOSE:
            bClosing = TRUE;  // Set closing flag
            bDraw = FALSE;    // Stop drawing
            DestroyWindow(hWnd);
            break;

        case WM_DESTROY:
            bClosing = TRUE;  // Ensure closing flag is set
            bDraw = FALSE;    // Ensure drawing is stopped
            PostQuitMessage(0);
            break;

        default:
            return DefWindowProc(hWnd, message, wParam, lParam);
    }
    return 0;
}

LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_INITDIALOG:
            return TRUE;

        case WM_COMMAND:
            if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
            {
                EndDialog(hDlg, LOWORD(wParam));
                return TRUE;
            }
            break;
    }
    return FALSE;
}


